package casino.agents;
import lime.*;
import lights.adapters.*;
import lights.interfaces.*;
import java.io.*;
import casino.plugins.*;
import casino.misc.*;
import casino.plugins.lookup.basic.*;
import casino.plugins.lookup.version1_0.*;
import casino.migration.*;


public class PluginBoard extends StationaryAgent {

	private LimeTupleSpace lts = null;
	private static final boolean DEBUG = true;
	private casino.misc.Queue opQueue;
	private boolean alive;

	public PluginBoard() {
		super();
	}

	private void initReaction() {

		enqueueOp(new Runnable() {public void run() {
		
			ITuple template = new Tuple();
			template.addFormal(Integer.class);
			// etc....
			try {
				lts.addWeakReaction(new Reaction[] { new UbiquitousReaction(template, new MessageListener(PluginBoard.this), Reaction.ONCEPERTUPLE)});
			} catch(TupleSpaceEngineException tse) {
				tse.printStackTrace();
			}
		}});
		if(DEBUG) showMessage("Reactions initialized");
	}

	public ITuple buildTuple(String name, Double version, Plugin plugin) {
		ITuple tuple = new Tuple();
		if(name != null) {
			tuple.addActual(name);
		} else {
			tuple.addFormal(String.class);
		}
		if(version != null) {		
			tuple.addActual(version);
		} else {
			tuple.addFormal(Double.class);
		}
		if(plugin != null) {
			PluginContainer container = new PluginContainer(plugin);
			tuple.addActual(container);
		} else {
			tuple.addFormal(PluginContainer.class);
		}
		return tuple;
	}	

	public void run() {
		opQueue = new Queue();
		try {
                        LimeTupleSpace lookup = new LimeTupleSpace(CommonPluginTerms.LOOKUP_TS_VERSION_1_0);
                        lookup.setShared(true);
                        
                        LimeServer.getServer().engage();
                        
                        lts = new LimeTupleSpace(CommonPluginTerms.PLUGINBOARD_TS_VERSION_1_0);
			lts.setShared(true);
                        
                        LULookupNachricht ln = new LULookupNachricht(lookup.rd(new LULookupNachricht(null,CommonPluginTerms.LOOKUP_TS_VERSION_1_0,-5).getRequestTuple()));
                        HostLocation lookupHost = ln.getPlace();
                        showMessage("Lookup-Host: "+ln.getPlace());
                        LUAdministratorNachricht an = new LUAdministratorNachricht(lookup.rd(new LUAdministratorNachricht(null,-5,CommonPluginTerms.LOOKUP_TS_VERSION_1_0,null).getRequestTuple()));
                        AgentLocation lookupPostbox = an.getPlace();
                        showMessage("Lookup-Administrator: "+an.getPlace());
                        LUIDNachricht idn = new LUIDNachricht(lookup.in(new LUIDNachricht(-42).getRequestTuple()));
                        long ID = idn.getID();
                        showMessage("Own ID: "+ID);
                        lookup.out(lookupPostbox,new LUAdministratorNachricht("PluginboardLeader",ID,CommonPluginTerms.PLUGINBOARD_TS_VERSION_1_0,new AgentLocation(getMgr().getID())).getResultTuple());
                        showMessage("Registered Administrator at Lookup.");
                        lookup.out(lookupPostbox,new LUPluginboardNachricht(ID,new HostLocation(LimeServer.getServer().getServerID()),CommonPluginTerms.PLUGINBOARD_TS_VERSION_1_0,1.0).getResultTuple());
                        showMessage("Registered Pluginboard at Lookup.");
                        lts.out(new AgentLocation(getMgr().getID()),new VersionNachricht(1.0).getResultTuple());
                        
                        
                        
                        
                        
                        
                        
			if(DEBUG) showMessage("PluginBoard-TS shared");
	
			initReaction();
			
			

			// My location for injection of additional plugins

			AgentLocation loc = new AgentLocation(this.getMgr().getID());
			ITuple locTuple = new Tuple();
			locTuple.addActual(new String("ADMINLOC"));
			locTuple.addActual(loc);
			lts.out(locTuple);
			showMessage("Wrote my PluginBoard-Admin-Location for additional Plugin injections");
                        
                        HostLocation hloc = new HostLocation(LimeServer.getServer().getServerID());
			ITuple hlocTuple = new Tuple();
			hlocTuple.addActual(new String("HOSTLOC"));
			hlocTuple.addActual(hloc);
			lts.out(hlocTuple);
			showMessage("Wrote my PluginBoard-Host-Location for additional Plugin injections");

			// For each Plugin-Type

			casino.plugins.lookup.PlayerLookupPlugin lpp = new casino.plugins.lookup.PlayerLookupPlugin();
			lts.out(buildTuple(casino.plugins.CommonPluginTerms.PLAYER_LOOKUP_PLUGIN,new Double(0),lpp));
			showMessage("Wrote LookupPlayerPlugin");

			casino.plugins.lookup.LeaderLookupPlugin llp = new casino.plugins.lookup.LeaderLookupPlugin();
			lts.out(buildTuple(casino.plugins.CommonPluginTerms.LEADER_LOOKUP_PLUGIN,new Double(0),llp));
			showMessage("Wrote LookupLeaderPlugin");

			casino.plugins.whiteboard.WhiteboardPlayerComponent wpc = new casino.plugins.whiteboard.WhiteboardPlayerComponent("Player");
			lts.out(buildTuple(casino.plugins.CommonPluginTerms.PLAYER_WHITEBOARD_PLUGIN,new Double(0),wpc));
			showMessage("Wrote WhiteBoardPlayerComponent");
				
			casino.plugins.whiteboard.WhiteboardLeaderComponent wlc = new casino.plugins.whiteboard.WhiteboardLeaderComponent("Leader");
			lts.out(buildTuple(casino.plugins.CommonPluginTerms.LEADER_WHITEBOARD_PLUGIN,new Double(0),wlc));
			showMessage("Wrote WhiteBoardLeaderComponent");

			casino.plugins.game.meier.MeierPlayer mp = new casino.plugins.game.meier.MeierPlayer();
			lts.out(buildTuple(casino.plugins.CommonPluginTerms.PLAYER_MEIER_PLUGIN,new Double(0),mp));
			showMessage("Wrote MeierPlayer");

			casino.plugins.game.meier.MeierStrategyPlugin msp = new casino.plugins.game.meier.MeierStrategyPlugin();
			lts.out(buildTuple(casino.plugins.CommonPluginTerms.MEIER_STRATEGY_PLUGIN,new Double(0),msp));
			showMessage("Wrote MeierStrategyPlugin");

			casino.plugins.game.meier.MeierGameLeader mgl = new casino.plugins.game.meier.MeierGameLeader();
			lts.out(buildTuple(casino.plugins.CommonPluginTerms.MEIER_LEADER_PLUGIN,new Double(0),mgl));
			showMessage("Wrote MeierGameLeader");

			casino.plugins.decision.MeierLeaderDecisionComponent mldc = new casino.plugins.decision.MeierLeaderDecisionComponent();
			lts.out(buildTuple(casino.plugins.CommonPluginTerms.LEADER_DECISION_PLUGIN,new Double(0),mldc));
			showMessage("Wrote MeierLeaderDecisionComponent");

			casino.plugins.decision.MeierPlayerDecisionComponent mpdc = new casino.plugins.decision.MeierPlayerDecisionComponent();
			lts.out(buildTuple(casino.plugins.CommonPluginTerms.PLAYER_DECISION_PLUGIN,new Double(0),mpdc));
			showMessage("Wrote MeierPlayerDecisionComponent");

                        lts.out(loc,(new MigrationToken()).free());
                        
			// End for each Plugin-Type
	
			// Finished loading initial Plugins	

			showMessage("PluginBoard initialized");

			while(true) {
                            try{
                                sleep(Long.MAX_VALUE);
                            }catch (InterruptedException e){
                            }
				/*Runnable op = (Runnable) opQueue.getNext();
				if(op == null) break;
				try{ op.run(); }
				catch(Exception e) { e.printStackTrace(); }*/
			}
		} catch(LimeException le) {
			le.printStackTrace();
		}
	}

	void enqueueOp(Runnable op) {
		showMessage("enqueueOp " + op.toString());
		opQueue.add(op);
	}

	public void processEvent(ITuple tuple) {
		// Do something
	}
        
        protected void showMessage(String out){
            System.out.println("PluginBoard: "+out);
        }
    

class MessageListener implements ReactionListener {
	private PluginBoard agent = null;
	public MessageListener(PluginBoard agent) {
		this.agent = agent;
	}

	public void reactsTo(ReactionEvent e) {
		if(DEBUG) showMessage("Processing event");
		agent.processEvent((ITuple)e.getEventTuple());
	}		
        
}

}
	
