package casino.agents.environment.observer;

/*
 * ObserverAgent.java
 *
 * Created on July 13, 2002, 2:28 AM
 */

/**
 *
 * @author  marcel
 */

import lime.*;
import lime.util.console.*;
import lights.adapters.*;
import lights.interfaces.*;
import java.io.*;
import casino.misc.*;
import casino.plugins.*;
import casino.plugins.lookup.basic.*;
import casino.plugins.lookup.version1_0.*;
import casino.plugins.whiteboard.basic.*;
import casino.plugins.whiteboard.Version1_0.*;

public class ObserverAgent extends StationaryAgent implements IConsoleProvider {
   
	static final boolean DEBUG = false; 
	private Queue opQueue;
	ObserverFrame frame = null;
	private boolean overview = false;
	private boolean finished = false;
	private LimeConsole console = null;
	private boolean lookupSet = false;
	
	LimeTupleSpace whiteboardTS = null;
	LimeTupleSpace lookupTS = null;
	LimeTupleSpace pluginTS = null;

	String whiteboardTSName = CommonPluginTerms.WHITEBOARD_TS_VERSION_1_0;
	String lookupTSName = CommonPluginTerms.LOOKUP_TS_VERSION_1_0;
	String pluginTSName = CommonPluginTerms.PLUGINBOARD_TS_VERSION_1_0;

	static final int LOOKUP_TEST_MESSAGE = 0;
	static final int LUADMINISTRATOR_MESSAGE = 1;
	static final int LUGAME_MESSAGE = 2;
	static final int LUGAMETYP_MESSAGE = 3;
	static final int LUID_MESSAGE = 4;
	static final int LULEADER_MESSAGE = 5;
	static final int LULOOKUP_MESSAGE = 6;
	static final int LUPLAYER_MESSAGE = 7;
	static final int LUPLUGINBOARD_MESSAGE = 8;
	static final int LUPOSTOFFICE_MESSAGE = 9;
	static final int LUWHITEBOARD_MESSAGE = 10;

	static final int WHITEBOARD_TEST_MESSAGE = 42;
	static final int WBGAMENACHRICHT_MESSAGE = 43;
	static final int PLUGINBOARD_PLUGIN = 84;

	public lime.util.console.LimeConsole getConsole() {
		return console;
	}

	public void setConsole(LimeConsole c) {
		this.console = c;
	}

	public void run() { 
		opQueue = new Queue();
       		frame = new ObserverFrame(this);
		initReaction();
       		frame.show();
       		frame.lookupPrint("Starting LookupObserver ...");
		frame.whiteBoardPrint("Starting WhiteboardObserver ...");
		frame.pluginPrint("Starting PluginObserver ...");

		try {
			whiteboardTS = new LimeTupleSpace(whiteboardTSName);
			lookupTS = new LimeTupleSpace(lookupTSName);
			pluginTS = new LimeTupleSpace(pluginTSName);
		} catch(LimeException le) {
			le.printStackTrace();
		}
		whiteboardTS.setShared(true);
		lookupTS.setShared(true);
		pluginTS.setShared(true);
		LimeServer.getServer().engage();
		if(DEBUG)System.out.println("Server engaged");
		while(true) {
			Runnable op = (Runnable) opQueue.getNext();
			if(DEBUG)System.out.println("Executing next queued Op");
			if(op == null) break;
			try{ op.run(); }
			catch(Exception e) {e.printStackTrace();}
			if(DEBUG)System.out.println("Executed queued Op");
		}
	}

	public void  getOverview() {
		enqueueOp(new Runnable() {public void run() {update();}});
	}

	void unshareAll() {
		if(DEBUG) System.out.println("Unsharing all TS");
		enqueueOp(new Runnable() {public void run() {unshare();}});
	}

	public boolean finished() {
		return finished;
	}

	private void unshare() {
		//pluginTS.setShared(false);
		//lookupTS.setShared(false);
		//whiteboardTS.setShared(false);
		//LimeServer.getServer().disengage();
		if(DEBUG)System.out.println("Unshared and Disengaged");
		finished = true;
	}

	private void initReaction() {

		// LUADMINISTRATOR-REACTIONS
		enqueueOp(new Runnable() {public void run() {
			ITuple template = new Tuple();
			template.addFormal(String.class);
			template.addFormal(String.class);
			template.addFormal(Long.class);
			template.addFormal(String.class);
			template.addFormal(AgentLocation.UNSPECIFIED.getClass());
			// ...
			try {
				lookupTS.addWeakReaction(new Reaction[] { new UbiquitousReaction(template, new LookupListener(ObserverAgent.this,LUADMINISTRATOR_MESSAGE, 4), Reaction.ONCEPERTUPLE)});
		} catch(TupleSpaceEngineException tse) {	
			tse.printStackTrace();
		}}});

		//LUGAME
		enqueueOp(new Runnable() {public void run() {
			ITuple template = new Tuple();
			template.addFormal(String.class);
			template.addFormal(String.class);
			template.addFormal(String.class);
			template.addFormal(HostLocation.UNSPECIFIED.getClass());
			template.addFormal(Long.class);
			template.addFormal(Double.class);
			try {
				lookupTS.addWeakReaction(new Reaction[] { new UbiquitousReaction(template, new LookupListener(ObserverAgent.this,LUGAME_MESSAGE,6), Reaction.ONCEPERTUPLE)});
		} catch(TupleSpaceEngineException tse) {	
			tse.printStackTrace();
		}}});

		//LUGAMETYP
		enqueueOp(new Runnable() {public void run() {
			ITuple template = new Tuple();
			template.addFormal(String.class);
			template.addFormal(String.class);
			template.addFormal(Double.class);
			template.addFormal(String.class);
			template.addFormal(String.class);
			template.addFormal(String.class);
			template.addFormal(String.class);

			try {
				lookupTS.addWeakReaction(new Reaction[] { new UbiquitousReaction(template, new LookupListener(ObserverAgent.this,LUGAMETYP_MESSAGE, 7), Reaction.ONCEPERTUPLE)});
		} catch(TupleSpaceEngineException tse) {	
			tse.printStackTrace();
		}}});

		// LUID
		enqueueOp(new Runnable() {public void run() {
			ITuple template = new Tuple();
			template.addFormal(String.class);
			template.addFormal(Double.class);

			try {
				lookupTS.addWeakReaction(new Reaction[] { new UbiquitousReaction(template, new LookupListener(ObserverAgent.this,LUID_MESSAGE,2), Reaction.ONCEPERTUPLE)});
		} catch(TupleSpaceEngineException tse) {	
			tse.printStackTrace();
		}}});

		//LULEADER
		enqueueOp(new Runnable() {public void run() {
			ITuple template = new Tuple();
			template.addFormal(String.class);
			template.addFormal(String.class);
			template.addFormal(Long.class);
			template.addFormal(HostLocation.UNSPECIFIED.getClass());
			template.addFormal(String.class);

			try {
				lookupTS.addWeakReaction(new Reaction[] { new UbiquitousReaction(template, new LookupListener(ObserverAgent.this,LULEADER_MESSAGE,5), Reaction.ONCEPERTUPLE)});
		} catch(TupleSpaceEngineException tse) {	
			tse.printStackTrace();
		}}});

		//LULOOKUP
		enqueueOp(new Runnable() {public void run() {
			ITuple template = new Tuple();
			template.addFormal(String.class);
			template.addFormal(HostLocation.UNSPECIFIED.getClass());
			template.addFormal(String.class);
			template.addFormal(Long.class);

			try {
				lookupTS.addWeakReaction(new Reaction[] { new UbiquitousReaction(template, new LookupListener(ObserverAgent.this,LULOOKUP_MESSAGE,4), Reaction.ONCEPERTUPLE)});
		} catch(TupleSpaceEngineException tse) {	
			tse.printStackTrace();
		}}});

		// LUPLAYER
		enqueueOp(new Runnable() {public void run() {
			/*ITuple template = new Tuple();
			template.addFormal(String.class);
			template.addFormal(String.class);
			template.addFormal(Long.class);
			template.addFormal(HostLocation.UNSPECIFIED.getClass());
			template.addFormal(String.class);*/
			ITuple template = new LUPlayerNachricht(null,-1,null,null).getRequestTuple();

			try {
				lookupTS.addWeakReaction(new Reaction[] { new UbiquitousReaction(template, new LookupListener(ObserverAgent.this,LUPLAYER_MESSAGE,5), Reaction.ONCEPERTUPLE)});
		} catch(TupleSpaceEngineException tse) {	
			tse.printStackTrace();
		}}});

		// LUPLUGINBOARD
		enqueueOp(new Runnable() {public void run() {
			ITuple template = new Tuple();
			template.addFormal(String.class);
			template.addFormal(Long.class);
			template.addFormal(HostLocation.UNSPECIFIED.getClass());
			template.addFormal(String.class);
			template.addFormal(Double.class);

			try {
				lookupTS.addWeakReaction(new Reaction[] { new UbiquitousReaction(template, new LookupListener(ObserverAgent.this,LUPLUGINBOARD_MESSAGE,5), Reaction.ONCEPERTUPLE)});
		} catch(TupleSpaceEngineException tse) {	
			tse.printStackTrace();
		}}});

		// LUPOSTOFFICE
		enqueueOp(new Runnable() {public void run() {
			ITuple template = new Tuple();
			template.addFormal(String.class);
			template.addFormal(Long.class);
			template.addFormal(HostLocation.UNSPECIFIED.getClass());
			template.addFormal(String.class);
			template.addFormal(Double.class);

			try {
				lookupTS.addWeakReaction(new Reaction[] { new UbiquitousReaction(template, new LookupListener(ObserverAgent.this,LUPOSTOFFICE_MESSAGE,5), Reaction.ONCEPERTUPLE)});
		} catch(TupleSpaceEngineException tse) {	
			tse.printStackTrace();
		}}});

		//LUWHITEBOARD
		enqueueOp(new Runnable() {public void run() {
			ITuple template = new Tuple();
			template.addFormal(String.class);
			template.addFormal(Long.class);
			template.addFormal(HostLocation.UNSPECIFIED.getClass());
			template.addFormal(String.class);
			template.addFormal(Double.class);

			try {
				lookupTS.addWeakReaction(new Reaction[] { new UbiquitousReaction(template, new LookupListener(ObserverAgent.this,LUWHITEBOARD_MESSAGE,5), Reaction.ONCEPERTUPLE)});
		} catch(TupleSpaceEngineException tse) {	
			tse.printStackTrace();
		}}});

/////////////////////////////////////////////////////////
		// WHITEBOAD-REACTIONS
		enqueueOp(new Runnable() {public void run() {
			ITuple template = new Tuple();
			template.addFormal(Integer.class);
			template.addFormal(Integer.class);
			// ...
			try {
				whiteboardTS.addWeakReaction(new Reaction[] { new UbiquitousReaction(template, new WhiteboardListener(ObserverAgent.this,WHITEBOARD_TEST_MESSAGE,2), Reaction.ONCEPERTUPLE)});
		} catch(TupleSpaceEngineException tse) {	
			tse.printStackTrace();
		}}});

		enqueueOp(new Runnable() {public void run() {
			ITuple template = new Tuple();
			template.addFormal(String.class);
			template.addFormal(String.class);
			template.addFormal(Double.class);
			template.addFormal(String.class);
			template.addFormal(Long.class);
			template.addFormal(Long.class);
			template.addFormal(String.class);
			// ...
			try {
				whiteboardTS.addWeakReaction(new Reaction[] { new UbiquitousReaction(template, new WhiteboardListener(ObserverAgent.this,WBGAMENACHRICHT_MESSAGE,7), Reaction.ONCEPERTUPLE)});
		} catch(TupleSpaceEngineException tse) {	
			tse.printStackTrace();
		}}});

		// Plugin-Load
		enqueueOp(new Runnable() {public void run() {
			ITuple template = new Tuple();
			template.addFormal(String.class);
			template.addFormal(Double.class);
			template.addFormal(PluginContainer.class);

			try {
				pluginTS.addWeakReaction(new Reaction[] { new UbiquitousReaction(template, new PluginListener(ObserverAgent.this,WBGAMENACHRICHT_MESSAGE,3), Reaction.ONCEPERTUPLE)});
		} catch(TupleSpaceEngineException tse) {	
			tse.printStackTrace();
		}}});

		if(DEBUG) System.out.println("Reactions initialized");
	}

	public void update() {
		
		// aus dem lookup service
		// update Admins
		frame.adminClear();
		try {
			LULookupNachricht ln = new LULookupNachricht(lookupTS.rd(new LULookupNachricht(null,CommonPluginTerms.LOOKUP_TS_VERSION_1_0,-5).getRequestTuple()));
			HostLocation lookupHost = ln.getPlace();
			ITuple cadmin = new Tuple();
			cadmin.addFormal(String.class);
			cadmin.addFormal(String.class);
			cadmin.addFormal(Long.class);
			cadmin.addFormal(String.class);
			cadmin.addFormal(AgentLocation.UNSPECIFIED.getClass());
			ITuple[] adminresults = lookupTS.rdg(lookupHost,AgentLocation.UNSPECIFIED,cadmin);
			if(adminresults!=null) {
				frame.adminUpdate("Found " + adminresults.length + " Admin(s):");
				for(int i=0;i<adminresults.length;i++) {
					frame.adminUpdate(adminresults[i].get(3).getValue() + " on " + adminresults[i].get(4).getValue());
				}
			} else frame.adminUpdate("No Admins");
		} catch(LimeException le) {le.printStackTrace();}			
		// update Players
		frame.playerClear();
		try {
			LULookupNachricht ln = new LULookupNachricht(lookupTS.rd(new LULookupNachricht(null,CommonPluginTerms.LOOKUP_TS_VERSION_1_0,-5).getRequestTuple()));
			HostLocation lookupHost = ln.getPlace();
			LUPlayerNachricht pn = new LUPlayerNachricht(null,-1,null,null);
			
			ITuple[] presults = lookupTS.rdg(lookupHost,AgentLocation.UNSPECIFIED,pn.getRequestTuple());
			if(presults!=null) {
				frame.playerUpdate("Found " + presults.length + " Player(s):");
				for(int i=0;i<presults.length;i++) {
					frame.playerUpdate("Player (ID: "+ presults[i].get(2).getValue()+")");
				}
			} else frame.playerUpdate("No Players");
		} catch(LimeException le) {le.printStackTrace();}			
		

		// update Leaders
		frame.leaderClear();
		try {
			LULookupNachricht ln = new LULookupNachricht(lookupTS.rd(new LULookupNachricht(null,CommonPluginTerms.LOOKUP_TS_VERSION_1_0,-5).getRequestTuple()));
			HostLocation lookupHost = ln.getPlace();
			ITuple ltemplate = new Tuple();
			LULeaderNachricht len = new LULeaderNachricht(null,-1,null,null);
			ITuple[] lresults = lookupTS.rdg(lookupHost,AgentLocation.UNSPECIFIED,len.getRequestTuple());
			if(lresults!=null) {
				frame.leaderUpdate("Found " + lresults.length + " Leader(s):");
				for(int i=0;i<lresults.length;i++) {
					frame.leaderUpdate("Leader (ID: "+ lresults[i].get(2).getValue()+")");
				}
			} else frame.leaderUpdate("No Leaders");
		} catch(LimeException le) {le.printStackTrace();}			
		

		// update Plugins
		frame.pluginClear();
		try {
			// hole HostLocation des Lookup
			LULookupNachricht ln = new LULookupNachricht(lookupTS.rd(new LULookupNachricht(null,CommonPluginTerms.LOOKUP_TS_VERSION_1_0,-5).getRequestTuple()));
			HostLocation lookupHost = ln.getPlace();
			// hole HostLocation von Whiteboard
			LUPluginboardNachricht pbn = new LUPluginboardNachricht(-1,null,null,-1);
			ITuple lupres = lookupTS.rdp(lookupHost,AgentLocation.UNSPECIFIED,pbn.getRequestTuple());
			if(DEBUG)System.out.println("Found Pluginboard-tuple");
			if(lupres!=null) {
				if(DEBUG)System.out.println("Found Pluginboard");
				LUPluginboardNachricht lpbrd = new LUPluginboardNachricht(lupres);
				ITuple template = new Tuple();
				template.addFormal(String.class);	
				template.addFormal(Double.class);
				template.addFormal(PluginContainer.class);
				ITuple[] results = pluginTS.rdg(lpbrd.getPlace(),AgentLocation.UNSPECIFIED,template);
				if(results!=null) {
					frame.pluginUpdate("Found " + results.length + " Plugin(s):");
					for(int i=0;i<results.length;i++) {
						frame.pluginUpdate(results[i].get(0).getValue() + " Version: " + results[i].get(1).getValue());
					}
				} else frame.pluginUpdate("No Plugins");
			} else {
				frame.pluginUpdate("No Pluginboard available");
			}
		} catch(LimeException le) {le.printStackTrace();}	
		/*ITuple template = new Tuple();
		template.addFormal(String.class);	
		template.addFormal(Double.class);
		template.addFormal(PluginContainer.class);
		ITuple hosttemp = new Tuple();
		hosttemp.addActual("HOSTLOC");
		hosttemp.addFormal(HostLocation.class);
		try {
			pluginTS.setShared(true);
			System.out.println("Plugin shared");
			ITuple host = pluginTS.rd(hosttemp);
			HostLocation pluginLocation = (HostLocation) host.get(1).getValue();
			System.out.println("Got host");
			ITuple[] results = pluginTS.rdg(pluginLocation,AgentLocation.UNSPECIFIED,template);
			if(results!=null) {
				if(DEBUG)frame.pluginUpdate("Found " + results.length + " Plugin(s):");
				for(int i=0;i<results.length;i++) {
					frame.pluginUpdate(results[i].get(0).getValue() + " Version: " + results[i].get(1).getValue());
				}
			} else frame.pluginUpdate("No Plugins");
		} catch(LimeException le) {le.printStackTrace();}*/

		frame.gamesPrintClear();
		try {
			// hole HostLocation des Lookup
			LULookupNachricht ln = new LULookupNachricht(lookupTS.rd(new LULookupNachricht(null,CommonPluginTerms.LOOKUP_TS_VERSION_1_0,-5).getRequestTuple()));
			HostLocation lookupHost = ln.getPlace();
			// hole HostLocation von Whiteboard
			LUWhiteboardNachricht lwb = new LUWhiteboardNachricht(-1,null,null,-1);
			ITuple lwbres = lookupTS.rdp(lookupHost,AgentLocation.UNSPECIFIED,lwb.getRequestTuple());
			if(lwbres!=null) {
				LUWhiteboardNachricht lwbrd = new LUWhiteboardNachricht(lwbres);
				// hole aus Whiteboard laufende Games
				WBGameNachricht wgn = new WBGameNachricht(null,null,-1,-1,-1,null);
				ITuple[] gresults = whiteboardTS.rdg(lwbrd.getPlace(),AgentLocation.UNSPECIFIED,wgn.getRequestTuple());
				if(gresults!=null) {
					frame.gamesPrint("Found " + gresults.length + " Game(s):");
					for (int i=0;i<gresults.length;i++) {
						frame.gamesPrint(gresults[i].get(1).getValue() + " Version: " + gresults[i].get(2).getValue().toString() + " Leader: " + gresults[i].get(3).getValue());
					}
				} else frame.gamesPrint("No Games");
			} else {
				frame.gamesPrint("No Whiteboard available");
			}
		} catch(LimeException le) {le.printStackTrace();}	
	}

	public void showGamesTupleSpace() {
		//start Console()
		if(DEBUG)System.out.println("Start Games Console");
	}

	public void showLookupTupleSpace() {
		if(DEBUG)System.out.println("Start Lookup Console");
		enqueueOp(new Runnable() {public void run() { setLookupConsole();}});
		if(DEBUG)System.out.println("Wait until LookupConsole is set");
		/*while(!lookupSet){
			try{sleep(1000);}catch(Exception e){;}
		}*/
		if(DEBUG)System.out.println("LookupConsole set");
		/*enqueueOp(new Runnable() {public void run() {
			while(true) {
				if(!console.isOpQueueEmpty()) {
					console.performQueuedOp();
				}
				try{sleep(5000);}catch(Exception e){;}
			}}});*/
		enqueueOp(new Runnable() {public void run() {
			consoleLoop();}});
	}
	
	private void consoleLoop() {
		new Thread(new Runnable() {public void run() {
			while(true) {
				if(!console.isOpQueueEmpty()) {
					console.performQueuedOp();
				}
				try{sleep(5000);}catch(Exception e){;}
			}}}).start();
	}	
		

	private void setLookupConsole() {
		console = new LimeConsole(this.getMgr().getID(),lookupTS,this);
		lookupSet = true;
	}	

	public void showWhiteboardTupleSpace() {
		if(DEBUG)System.out.println("Start Whiteboard Console");
	}

	public void showPluginTupleSpace() {
		if(DEBUG)System.out.println("Start Plugin Console");
	}

	void enqueueOp(Runnable op) {
		if(DEBUG) System.out.println("enqueueOp " + op.toString());
		opQueue.add(op);
	}

	public void processLookupEvent(ITuple tuple, int type, int length) {
		if(DEBUG)frame.lookupPrint("Reacting to type " + type);
		String message = null;
		switch(type) {
			case LUADMINISTRATOR_MESSAGE:
				message = " --> LUADMINISTRATOR:";
				for(int i = 0; i<length;i++) {
					message += ":";
					if((tuple.get(i).getValue())!=null) {
						message += (String)tuple.get(i).getValue().toString();
					} else {message += "null";}
				}
				frame.lookupPrint(message);
				break;
			case LUGAME_MESSAGE:
				message = " --> LUGAME:";
				for(int i = 0; i<length;i++) {
					message += ":";
					if((tuple.get(i).getValue())!=null) {
						message += (String)tuple.get(i).getValue().toString();
					} else {message += "null";}
				}
				frame.lookupPrint(message);
				break;
			case LUGAMETYP_MESSAGE:
				message = " --> LUGAMETYP:";
				for(int i = 0; i<length;i++) {
					message += ":";
					if((tuple.get(i).getValue())!=null) {
						message += (String)tuple.get(i).getValue().toString();
					} else {message += "null";}
				}
				frame.lookupPrint(message);
				break;
			case LUID_MESSAGE:
				message = " --> LUID:";
				for(int i = 0; i<length;i++) {
					message += ":";
					if((tuple.get(i).getValue())!=null) {
						message += (String)tuple.get(i).getValue().toString();
					} else {message += "null";}
				}
				frame.lookupPrint(message);
				break;
			case LULEADER_MESSAGE:
				message = " --> LULEADER:";
				for(int i = 0; i<length;i++) {
					message += ":";
					if((tuple.get(i).getValue())!=null) {
						message += (String)tuple.get(i).getValue().toString();
					} else {message += "null";}
				}
				frame.lookupPrint(message);
				break;
			case LULOOKUP_MESSAGE:
				message = " --> LULOOKUP:";
				for(int i = 0; i<length;i++) {
					message += ":";
					if((tuple.get(i).getValue())!=null) {
						message += (String)tuple.get(i).getValue().toString();
					} else {message += "null";}
				}
				frame.lookupPrint(message);
				break;
			case LUPLAYER_MESSAGE:
				message = " --> LUPLAYER:";
				for(int i = 0; i<length;i++) {
					message += ":";
					if((tuple.get(i).getValue())!=null) {
						message += (String)tuple.get(i).getValue().toString();
					} else {message += "null";}
				}
				frame.lookupPrint(message);
				break;
			case LUPLUGINBOARD_MESSAGE:
				message = " --> LUPLUGINBOARD:";
				for(int i = 0; i<length;i++) {
					message += ":";
					if((tuple.get(i).getValue())!=null) {
						message += (String)tuple.get(i).getValue().toString();
					} else {message += "null";}
				}
				frame.lookupPrint(message);
				break;
			case LUPOSTOFFICE_MESSAGE:
				message = " --> LUPOSTOFFICE:";
				for(int i = 0; i<length;i++) {
					message += ":";
					if((tuple.get(i).getValue())!=null) {
						message += (String)tuple.get(i).getValue().toString();
					} else {message += "null";}
				}
				frame.lookupPrint(message);
				break;
			case LUWHITEBOARD_MESSAGE:
				message = " --> LUWHITEBOARD:";
				for(int i = 0; i<length;i++) {
					message += ":";
					if((tuple.get(i).getValue())!=null) {
						message += (String)tuple.get(i).getValue().toString();
					} else {message += "null";}
				}
				frame.lookupPrint(message);
				break;
		}
	}

	public void processWhiteboardEvent(ITuple tuple, int type, int length) {
		if(DEBUG)frame.whiteBoardPrint("Reacting to type " + type);
		switch(type) {
			case WBGAMENACHRICHT_MESSAGE:
				String message = " --> WBGAME :";
				for(int i = 0; i<length;i++) {
					message += ":";
					if((tuple.get(i).getValue())!=null) {
						message += (String)tuple.get(i).getValue().toString();
					} else {message += "null";}
				}
				frame.whiteBoardPrint(message);
				break;
		}
	}

	public void processPluginEvent(ITuple tuple, int type, int length) {
		if(DEBUG)frame.pluginPrint("Reacting to type " + type);
				String message = "Loaded Plugin: ";
				for(int i = 0; i<length;i++) {
					message += ":";
					if((tuple.get(i).getValue())!=null) {
						message += (String)tuple.get(i).getValue().toString();
					} else {message += "null";}
				}
				frame.pluginPrint(message);
	}

   /**
     * @param args the command line arguments
     */
    public static void main(String args[]) {
        new lime.util.Launcher().launch(args,0);
		try {
				LimeServer.getServer().loadAgent("casino.agents.environment.observer.ObserverAgent",new String[] {});
	//			LimeServer.getServer().loadAgent("casino.agents.environment.observer.TestAgent",new String[] {});
		} catch (LimeException le) {
				le.printStackTrace();
		}
		
    }
    
class LookupListener implements ReactionListener {
	private ObserverAgent agent = null;
	private int type;
	private int length;
	public LookupListener(ObserverAgent agent, int type, int length) {
		this.agent = agent;
		this.type = type;
		this.length = length;
	}

	public void reactsTo(ReactionEvent e) {
		if(DEBUG) System.out.println("Lookup-Reaction");
		agent.processLookupEvent((ITuple)e.getEventTuple(), type,length);
	}
        
}   

class WhiteboardListener implements ReactionListener {
	private ObserverAgent agent = null;
	private int type;
	private int length;
	public WhiteboardListener(ObserverAgent agent, int type, int length) {
		this.agent = agent;
		this.type = type;
		this.length = length;
	}

	public void reactsTo(ReactionEvent e) {
		if(DEBUG) System.out.println("Whiteboard-Reaction");
		agent.processWhiteboardEvent((ITuple)e.getEventTuple(), type,length);
	}
        
}   

class PluginListener implements ReactionListener {
	private ObserverAgent agent = null;
	private int type;
	private int length;
	public PluginListener(ObserverAgent agent, int type, int length) {
		this.agent = agent;
		this.type = type;
		this.length = length;
	}

	public void reactsTo(ReactionEvent e) {
		if(DEBUG) System.out.println("Plugin-Reaction");
		agent.processPluginEvent((ITuple)e.getEventTuple(), type,length);
	}
        
}   
}
