/*
 * OpQueue.java
 *
 * Created on 16. Juni 2002, 09:48
 */

package casino.misc;

import java.lang.reflect.*;
import java.util.*;

import lime.*;

/**
 *
 * @author  sven
 * @version 1.0
 */
public class OpQueue implements java.io.Serializable{
    
    protected LimeTupleSpace objectToInvokeOn;
    protected java.util.Vector methods;
    protected java.util.Vector results;
    protected java.util.Vector javaPrimitives;
    protected long IDCounter = -1;
    
    /** Creates new OpQueue */
    public OpQueue(LimeTupleSpace objectToInvokeOn) {
        this.objectToInvokeOn = objectToInvokeOn;
        IDCounter = 0;
        methods = new Vector();
        results = new Vector();
        javaPrimitives = new Vector();
        javaPrimitives.add(Boolean.class);
        javaPrimitives.add(Integer.class);
        javaPrimitives.add(Long.class);
        javaPrimitives.add(Float.class);
        javaPrimitives.add(Double.class);
    }

    public synchronized Object enqueueOp(String methodName, Object[] parameters) throws NoSuchMethodException{
        
        Method method = null;
        boolean resultAvailable = false;
        
        Class retrieveMethod = objectToInvokeOn.getClass();
        
        //Klassen der Parameter fr die Methode methodName bestimmen
        
        Method[] availableMethods = objectToInvokeOn.getClass().getMethods();
        Class[] parameterClasses = new Class[parameters.length];
        boolean methodFound = false;
        for (int i = 0; i < availableMethods.length  && (!methodFound); i++){
            Class[] currentParameters = availableMethods[i].getParameterTypes();
            if ((availableMethods[i].getName().equals(methodName)) && (currentParameters.length == parameters.length)){
                if (matches(currentParameters, parameters)){
                    parameterClasses = availableMethods[i].getParameterTypes();
                    methodFound = true;
                }
            }
        }
        if (!methodFound){
            throw new NoSuchMethodException();
        }
        
        //Methodennamen methodName in Methode umwandeln mit Hilfe von java.lang.reflect
        try{
            method = retrieveMethod.getMethod(methodName, parameterClasses);
        }catch (NoSuchMethodException e){
            System.out.println("Exception in OpQueue method "+methodName+"("+parameterClasses+") does not exist: "+e.getMessage());
            throw e;
        }
        //Methode zum Speicher vorbereiten
        MethodToInvoke m = new MethodToInvoke(IDCounter, method, parameters);
        //und dann speichern.
        methods.add(m);
        //nchste ID setzen.
        IDCounter++;
        //Auf Methodenausfhrung wartende Thread benachrichtigen
        //notifyAll();
        OpResult resultOfMethod = null;
        //Solange Methode noch nicht ausgefhrt.
        while (!resultAvailable){
            //gucken, ob Methode ausgefhrt wurde
            //Alle Ergebnisse durchlaufen
            for (int i = 0; i < results.size(); i++){
                //Das Result mit der passenden ID suchen
                OpResult currentResult = (OpResult)results.get(i);
                if (currentResult.getResultID() == m.getMethodID()){
                    resultOfMethod = currentResult;
                    //Ergebnis ist da!
                    resultAvailable = true;
                    //Ergebnis aus ResultQueue entfernen.
                    results.remove(currentResult);
                }
            }
            //Wenn Ergebnis nicht da,
            if (!resultAvailable) {
                //warten bis neue Ergebnisse da.
                try {
                    wait();
                }catch (InterruptedException e){
                    e.printStackTrace();
                }
            }
        }
        //Methode methodName wurde beendet und Ergebnis kann zurckgegeben werden.
        //System.out.println("OpQueue: returning "+resultOfMethod.getResult());
        return resultOfMethod.getResult();
    }
    
    //Methode zum Aufrufen der nchsten Methode in der Queue
    public synchronized void executeNextQueuedOp() {
        
        OpResult invocationResult = null;
        
        //Solange keine Methoden zum Aufruf da sind
        /*while (methods.size() == 0){
            //warten bis Methode da.
            try{
                wait();
            }catch (InterruptedException e){
                System.out.println("Exception in OpQueue: ");
                e.printStackTrace();
            }
        }*/
        //Erste Methode aus der Queue nehmen.
        if (!(methods.size() == 0)){
            MethodToInvoke m = (MethodToInvoke)methods.remove(0);
            try{
                //Das Result erstellen mit entsprechender ID und Aufruf der Methode.
                invocationResult = new OpResult(m.getMethodID(), m.getMethod().invoke(objectToInvokeOn, m.getParameters()));
            }catch (IllegalAccessException e){
                System.out.println("Exception in OpQueue: ");
                e.printStackTrace();
            }catch (InvocationTargetException e){
                System.out.println("Exception in OpQueue: ");
                e.printStackTrace();
            }
            //Result in result-Queue speichern
            results.add(invocationResult);
            //Alle auf Ergebnisse wartende Threads aufwecken
            notifyAll();
        }
    }
    
    private boolean matches(Class[] parameters, Object[] classesToMatch){

        if (parameters.length == classesToMatch.length){
            boolean matches = true;
            for (int i = 0; (i < parameters.length) && (matches); i++){

                if (matches){

                    boolean oneClassMatches = false;
                    Vector typesOfCurrentClass = new Vector();
                    Class currentType = classesToMatch[i].getClass();
                    Class objectType = new Object().getClass();
                    
                    //feststellen zu welchen Klassen classesToMatch[i] gehrt
                    while (!(objectType.equals(currentType))){
                        typesOfCurrentClass.add(currentType);
                        Class[] interfaces = classesToMatch[i].getClass().getInterfaces();
                        for (int j = 0; j < interfaces.length; j++){
                            typesOfCurrentClass.add(interfaces[j]);
                        }
                        currentType = currentType.getSuperclass();
                    }
                    
                    Class[] currentClasses = new Class[typesOfCurrentClass.size()];
                    for (int j = 0; j < typesOfCurrentClass.size(); j++){
                        currentClasses[j] = (Class)typesOfCurrentClass.get(j);
                    }
                    
                    int index = 0;
                    while ( (!oneClassMatches) && (typesOfCurrentClass.size() > index)){
                        //System.out.println("OpQueue: Testing if "+parameters[i]+" matches "+typesOfCurrentClass.get(index)+".");
                        if ((parameters[i].equals(typesOfCurrentClass.get(index))) || javaPrimitives.contains(typesOfCurrentClass.get(index))){
                            oneClassMatches = true;
                            //System.out.println("OpQueue: "+parameters[i]+" matches "+typesOfCurrentClass.get(index)+".");
                        }
                        index++;
                    }
                    if (!oneClassMatches){
                        matches = false;
                    }
                }
            }
            return matches;
        }else{
            return false;
        }
    }
    
    //Hilfsklasse, die eine aufzurufende Methode aufnimmt.
    protected class MethodToInvoke{
        
        protected long methodID;
        protected Method method;
        protected Object[] parameters;
        
        public MethodToInvoke(long methodID, Method method, Object[] parameters){
            this.methodID = methodID;
            this.method = method;
            this.parameters = parameters;
        }
        
        public long getMethodID(){
            return methodID;
        }
        
        public Method getMethod(){
            return method;
        }
        
        public Object[] getParameters(){
            return parameters;
        }
    }
    
    //Hilfsklasse, die das Ergebnisse eines MethodenAufrufs aufnimmt.
    protected class OpResult{
        
        private long resultID = -1;
        private Object result;
        
        public OpResult(long resultID, Object result){
            this.resultID = resultID;
            this.result = result;
        }
        
        public long getResultID(){
            return resultID;
        }
        
        public Object getResult(){
            return result;
        }
    } 

    public boolean isEmpty(){
        return (this.methods.size() == 0);
    }
    
}