/*
 * DecisionComponent.java
 *
 * Created on 21. Juni 2002, 18:49
 */

package casino.plugins.decision;

import casino.agents.*;
import casino.plugins.*;
import casino.plugins.game.meier.*;
import casino.plugins.whiteboard.*;
import casino.plugins.lookup.*;
import casino.misc.*;

import lime.*;
import lights.interfaces.*;
import lights.adapters.*;

import java.util.Hashtable;
import java.util.Enumeration;

/**
 *
 * @author  jens
 * @version 
 */
public class MeierPlayerDecisionComponent extends Thread implements DecisionPlugIn {
    
    public static final double VERSION = 1.0;
    
    protected static final int JUST_CREATED = 0;
    public static final int READY = 1;
    protected static final int GAME_FOUND = 2;
    protected static final int NO_GAME_AVAILABLE = 3;
    public static final int GAME_HOST_REACHED = 4;
    
    public static final String LOOKUP = CommonPluginTerms.PLAYER_LOOKUP_PLUGIN;
    public static final String WHITEBOARD = CommonPluginTerms.PLAYER_WHITEBOARD_PLUGIN;
    public static final String MEIER = CommonPluginTerms.PLAYER_MEIER_PLUGIN;
    
    public static final int GAME_LOGON_DENIED = 0;
    public static final int GAME_REGULAR_END = 1;
    public static final int GAME_DID_NOT_START = 2;
    
    protected String name = null;
    protected long ID = ID_NOT_SET;
    protected CasinoAgent agent = null;
    protected int state = JUST_CREATED;
    protected boolean active = true;
    protected Hashtable plugins = null;
    protected HostLocation migrationHost = null;
    protected boolean readyForMigration = false;
    protected LimeTupleSpace lts = null;
    protected OpQueue operationQueue = null;
    protected boolean justMigrated = false;
    
    protected boolean readyToPlay = false;
    protected int lastGameEndState = GAME_DID_NOT_END;
    protected int gamesToPlay = 10;
    protected int gamesPlayed = 0;

    public static final int NO_GAME_FOUND = 3;
    public static final int GAME_DID_NOT_END = -1;
    
    /** Creates new DecisionComponent */
    public MeierPlayerDecisionComponent() {
    }
    
    public void run(){
        
        readyForMigration = false;
        migrationHost = null;
        if (!justMigrated){
                /*while(active){
                    switch(state){
                        case(JUST_CREATED) : initialize();state = READY;break;
                        case(READY) : 
                            showMessage("Suche Spiel!");
                            seekGame();
                            if(migrationHost != null){
                                state = GAME_FOUND;
                            }
                            else{
                                state = NO_GAME_AVAILABLE;
                            }
                            break;
                        case(NO_GAME_AVAILABLE) :  
                            state = READY;break;
                        case(GAME_FOUND) : active = false;break;
                    }
                }*/
            initialize();
            active = true;
            try{
                sleep(1000L);
            }catch (InterruptedException e){
                e.printStackTrace();
            }

            // Teste Status
            seekGame();
            while (lastGameEndState == NO_GAME_FOUND){
                seekGame();
                if (lastGameEndState == NO_GAME_FOUND){
                    showMessage("No game found. Waiting a few seconds");
                    try{
                        sleep(100000L);
                    }catch (InterruptedException e){
                        e.printStackTrace();
                    }
                }
            }
            lastGameEndState = GAME_DID_NOT_END;

            // Host gefunden 
            //HostLocation migrationHost = null;
            // Deaktiviere Plugins
            
            if(migrationHost != null){
                // Migriere
                /*if (migrationHost.equals(new HostLocation(agent.getLimeServer().getServerID())))*/ plugins.put(CommonPluginTerms.PLAYER_MEIER_PLUGIN, ((MeierPlayerRulePlugin)getPlugin(CommonPluginTerms.PLAYER_MEIER_PLUGIN, 1.0)).getNewInstance());
                showMessage("Migriere zu: "+migrationHost);
                readyToPlay = true;
                lts = null;
                //this.operationQueue = null;
                agent.migrate(migrationHost);
                justMigrated = true;
            }
        }else{ //gerade migriert;
            showMessage("Just migrated: Starting Meier...");
            lastGameEndState = GAME_DID_NOT_END;
            initialize();
            active = true;
            seekGame();
            //Warte solange das Spiel noch nicht beendet ist.
            while (lastGameEndState == GAME_DID_NOT_END){
                try{
                    sleep(10000L);
                }catch (InterruptedException e){
                    e.printStackTrace();
                }
            }
            readyToPlay = false;
            
            //Wenn ich mich nicht einloggen konnte, dann warten.
            if (lastGameEndState == this.GAME_LOGON_DENIED){
                showMessage("Logon to game denied. Waiting a few seconds");
                try{
                    sleep(10000L);
                }catch (InterruptedException e){
                    e.printStackTrace();
                }
            }
            if (lastGameEndState == this.GAME_DID_NOT_START){
                showMessage("Game did not start. Not enough players arrived");
            }
            if (lastGameEndState == this.GAME_REGULAR_END){
                showMessage("Game ended regularly");
                gamesPlayed++;
            }
            lastGameEndState = this.GAME_DID_NOT_END;
            //Wenn noch nicht Anzahl der gewnschten Spiele gemacht, dann neues Spiel suchen
            if (gamesPlayed < this.gamesToPlay) {
                showMessage("Try to find new game");
                try{
                    sleep(1000);
                }catch(Exception e){}
                seekGame();
                while (lastGameEndState == NO_GAME_FOUND){
                    seekGame();
                    if (lastGameEndState == NO_GAME_FOUND){
                        showMessage("No game found. Waiting a few seconds");
                        try{
                            sleep(5000L);
                        }catch (InterruptedException e){
                            e.printStackTrace();
                        }
                    }
                }
                lastGameEndState = this.GAME_DID_NOT_END;
                
                
                if(migrationHost != null){
                    // Migriere
                    /*if (migrationHost.equals(new HostLocation(agent.getLimeServer().getServerID())))*/ plugins.put(CommonPluginTerms.PLAYER_MEIER_PLUGIN, ((MeierPlayerRulePlugin)getPlugin(CommonPluginTerms.PLAYER_MEIER_PLUGIN, 1.0)).getNewInstance());
                    showMessage("Migriere zu: "+migrationHost);
                    readyToPlay = true;
                    lts = null;
                    //this.operationQueue = null;
                    agent.migrate(migrationHost);
                    justMigrated = true;
                }
            }else{
                //Sauber beenden
                showMessage("Enough games played. Leaving System and shutting down TupleSpaces.");
                Enumeration enum = plugins.elements();
                while(enum.hasMoreElements()){
                    Plugin plugin = (Plugin)enum.nextElement();
                    plugin.deactivate();
                }

                while (!this.migratable()){
                    try {
                        sleep(10000L);
                    }catch (InterruptedException e){
                        e.printStackTrace();
                    }
                }
                showMessage("Leaving System. Number of games played: "+gamesPlayed);
                active = false;
            }
            
        }
        showMessage("OpQueue auf null setzen!");
        lts = null;
        operationQueue = null;
        
    }
    
    public void setState(int newState){
        state = newState;
    }
            
    public boolean migratable(){
        Enumeration enum = plugins.elements();
        boolean rueckgabe = true;
        while(enum.hasMoreElements()){
            Plugin plugin = ((Plugin)enum.nextElement());
            //showMessage("Plugin vorhganden: "+plugin.getTyp());
            if((!plugin.isMigratable()) && !(plugin.getTyp().equals(CommonPluginTerms.PLAYER_LOOKUP_PLUGIN))){
                rueckgabe = false;
                showMessage("Not migratable plugin: "+plugin.getTyp());
            }
        }
        return rueckgabe;
    }
    
    protected void seekGame(){
        //WhiteboardPlayerPlugin wpp = (WhiteboardPlayerPlugin)getPlugin(WHITEBOARD,1.0);
        MeierPlayer mpp = (MeierPlayer)getPlugin(MEIER,1.0);
        //migrationHost = wpp.seekGame(CommonPluginTerms.MEIER_GAME,mpp.getVersion());
        mpp.activate();
    }
    
    public Plugin getPlugin(String PluginName,double pluginVersion){
        Plugin rueckgabe = (Plugin)plugins.get(PluginName);
        if((rueckgabe != null) && (((long)rueckgabe.getVersion() != (long)pluginVersion)||(rueckgabe.getVersion() < pluginVersion))){
            plugins.remove(PluginName);
            rueckgabe = null;
        }
        if(rueckgabe == null){
            ITuple lookupTuple = new Tuple();
            lookupTuple.addActual(PluginName);
            lookupTuple.addActual(new Double(pluginVersion));
            lookupTuple.addFormal(PluginContainer.class);
            try{
                rueckgabe = ((PluginContainer)operationQueue.enqueueOp("rdp",new Object[]{agent.getHost(),agent.getAdmin(),lookupTuple})).getPlugin(); 
            }catch(Exception e){e.printStackTrace();}
            plugins.put(PluginName,rueckgabe);
        }
        return rueckgabe;
    }
    
    protected void initialize(){
        if(plugins == null){
            plugins = new Hashtable();
            Plugin lookup = null;
            ITuple lookupTuple = new Tuple();
            lookupTuple.addActual(CommonPluginTerms.PLAYER_LOOKUP_PLUGIN);
            lookupTuple.addActual(new Double(0));
            lookupTuple.addFormal(PluginContainer.class);
            try{
                showMessage("Host: "+agent.getHost());
                showMessage("Agent: "+agent.getAdmin());
                PluginContainer clookup = (PluginContainer) ((ITuple)operationQueue.enqueueOp("rdp",new Object[]{agent.getHost(),agent.getAdmin(),lookupTuple})).get(2).getValue(); 
                lookup = clookup.getPlugin();
            }catch(Exception e){e.printStackTrace();}
            if(lookup != null){
                plugins.put(LOOKUP,lookup);
                ((LookupPlayerPlugin)lookup).setAgent(this);
                lookup.activate();
                while(!lookup.isActive()){
                    try{
                        sleep(100L);
                    }catch (InterruptedException e){
                        e.printStackTrace();
                    }
                }
            }
            showMessage("Lookup aktiviert!");
            Plugin whiteboard = null;
            ITuple whiteboardTuple = new Tuple();
            whiteboardTuple.addActual(CommonPluginTerms.PLAYER_WHITEBOARD_PLUGIN);
            whiteboardTuple.addActual(new Double(0));
            whiteboardTuple.addFormal(PluginContainer.class);
            try{
                PluginContainer wlookup = (PluginContainer) ((ITuple)operationQueue.enqueueOp("rdp",new Object[]{agent.getHost(),agent.getAdmin(),whiteboardTuple})).get(2).getValue(); 
                whiteboard = wlookup.getPlugin();
            }catch(Exception e){e.printStackTrace();}
            if(whiteboard != null){
                plugins.put(WHITEBOARD, whiteboard);
                ((WhiteboardPlayerPlugin)whiteboard).setLeader(this);
                showMessage("Aktiviere Whiteboard!");
                whiteboard.activate();
                while(!whiteboard.isActive()){
                //showMessage("Whiteboard not active!");
                    try{
                        sleep(100L);
                    }catch (InterruptedException e){
                        e.printStackTrace();
                    }
                }
            }
            Plugin meier = null;
            ITuple meierTuple = new Tuple();
            meierTuple.addActual(CommonPluginTerms.PLAYER_MEIER_PLUGIN);
            meierTuple.addActual(new Double(0));
            meierTuple.addFormal(PluginContainer.class);
            try{
                PluginContainer mlookup = (PluginContainer) ((ITuple)operationQueue.enqueueOp("rdp",new Object[]{agent.getHost(),agent.getAdmin(),meierTuple})).get(2).getValue(); 
                meier  = mlookup.getPlugin();
            }catch(Exception e){e.printStackTrace();}
            if(meier != null){
                plugins.put(MEIER,meier);
                ((MeierPlayer)meier).setDecisionPlugin(this);
                /*meier.activate();
                while(!meier.isActive()){

                }*/
            }
        }else{
            Enumeration enum = plugins.elements();
            ((Plugin)plugins.get(CommonPluginTerms.PLAYER_LOOKUP_PLUGIN)).activate();
            while(enum.hasMoreElements()){
                Plugin plugin = ((Plugin)enum.nextElement());
                showMessage("MeierPlayer DecisionComponent: Activating plugin: "+plugin.getTyp());
                if((plugin.getTyp() != casino.plugins.CommonPluginTerms.PLAYER_MEIER_PLUGIN) && (plugin.getTyp() != CommonPluginTerms.PLAYER_LOOKUP_PLUGIN)){
                    if (!plugin.isActive()){
                        //plugin.setAgent(this);
                        plugin.activate();
                        while(!plugin.isActive()){
                            try{
                                showMessage("Waiting for Plugin to start: "+plugin.getTyp());
                                sleep(2000);
                            }catch(Exception e){}
                        }
                        showMessage("Plugin started: "+plugin.getTyp());
                    }
                }
            }
        }
        showMessage("Initialisierung Player-Decision-Komponente abgeschlossen!");
    }
    
    public String getPluginName() {
        return name;
    }
    
    public void activate() {
        this.start();
    }
    
    public long getID() {
        return ID;
    }
    
    public void setTupleSpace(LimeTupleSpace TS) {
        lts = TS;
        operationQueue = new OpQueue(lts);
        
    }
    
    public void executeQueuedOp() {
        operationQueue.executeNextQueuedOp();
    }
    
    public void setCommAgent(casino.agents.CommAgent com) {
        // Kein Effekt, da hart mit Agent verdrahtet
    }
    
    public double getVersion() {
        return VERSION;
    }
    
    public void setID(long newID) {
        ID = newID;
    }
    
    public String getTyp() {
        return TYP;
    }
    
    public void setAgent(CasinoAgent newAgent) {
        agent = newAgent;
        if ( (agent != null) && (ID == ID_NOT_SET) ){
            this.ID = agent.getID();
        }
    }
    
    public CasinoAgent getAgent() {
        return agent;
    }
         
    public void deactivate(){
        
        Enumeration enum = plugins.elements();
        while(enum.hasMoreElements()){
            Plugin plugin = (Plugin)enum.nextElement();
            if ( !plugin.getTyp().equals(CommonPluginTerms.PLAYER_LOOKUP_PLUGIN) ){
                plugin.deactivate();
                while(plugin.isActive() == true){
                    try{sleep(10000);System.out.println("Waiting for Plugin "+plugin.getTyp()+ " to close!");}catch(Exception e){}
                }
            }
        }
        active = false;
    }
    
    public String getMainGame() {
        return null;
    }
    
    public boolean isActive(){
        return active;
    }
    
    public LimeServer getLimeServer() {
        return agent.getLimeServer();
    }
    
    public void activate(String TSName) {
        // Unsinnig!
    }
    
    public void setOpQueue(OpQueue newOpQueue) {
        operationQueue = newOpQueue;
    }
    
    public void setMigrationHost(HostLocation host) {
        this.migrationHost = host;
    }
    
    public boolean readyToPlay() {
        return readyToPlay;
    }
    
    public void notifyEndOfGame(int endState) {
        lastGameEndState = endState;
    }
    
    public boolean isMigratable() {
        return migratable();
    }
    
    protected void showMessage(String out){
        System.out.println("MeierPlayerDecision "+this.ID+": "+out);
    }
    
}
