/*
 * MeierPlayer.java
 *
 * Created on 17. Juni 2002, 22:15
 */

package casino.plugins.game.meier;

import java.net.InetAddress;
import java.net.UnknownHostException;
import java.io.Serializable;

import lime.RegisteredReaction;
import lights.adapters.*;
import lights.interfaces.*;
import lime.mobileagent.mucode.*;
import lime.HostLocation;
import lime.AgentLocation;
import lime.LimeServer;
import lime.LimeServerID;
import lime.ReactionListener;
import lime.Reaction;
import lime.UbiquitousReaction;
import lime.AgentID;

import casino.misc.OpQueue;
import casino.misc.MeierDice;
import casino.misc.Timer;
import casino.plugins.game.GamePlayerStrategyPlugin;
import casino.terms.game.meier.MeierTerms;
import casino.agents.CommAgent;
import casino.plugins.decision.*;
import casino.plugins.*;
import casino.plugins.whiteboard.*;
import casino.plugins.whiteboard.basic.*;
import casino.plugins.whiteboard.Version1_0.*;
import casino.plugins.lookup.*;
import casino.plugins.lookup.basic.*;
import casino.plugins.lookup.version1_0.*;
import casino.plugins.game.meier.gui.*;

/**
 *
 * @author  administrator
 * @version 1.0
 */
public class MeierPlayer extends Thread implements MeierPlayerRulePlugin {
    
    protected static final String pluginName = "Meier";
    protected static final double version = 1.0;
    
    protected boolean playing = false;
    protected boolean observing = false;
    protected boolean textMode = false;
    protected boolean loggedOn = false;
    protected boolean slowMo = true;
    
    protected long timeToWaitForLogon = 300000L;
    protected long leaderID = Long.MAX_VALUE;
    protected long slowMoFactor = 0L;
    
    protected int myPoints = 0;
    
    protected RegisteredReaction[] reactions = null;
    protected OpQueue opQueue = null;
    protected CommAgent chief = null;    
    protected MeierTerms terms = null;
    protected MeierPlayerStrategyPlugin strategy = null;
    protected DecisionPlugIn decisionMaker = null;
    
    protected HostLocation myLocation = null;
    protected AgentLocation leaderLocation = null;
    protected ITuple pointAssignmentToFetch = null;
    protected String TStoUse = null;
    protected MeierPlayerGUI gui = null;
    
    protected boolean migratable = true;

    /** Creates new MeierPlayer */
    public MeierPlayer() {
        /*try{
            myLocation = new HostLocation(new LimeServerID(InetAddress.getLocalHost(), LimeServer.getServer().getPort()));
        }catch (UnknownHostException e){
            showMessage("Exception occured: ");
            e.printStackTrace();
        }*/
        strategy = new MeierStrategyPlugin();
        terms = new MeierTerms();
    }

    public void observe() {
        observing = true;
        playing = false;
    }
    
    public void activate(){
        if (!decisionMaker.readyToPlay()){
            WhiteboardPlayerPlugin whiteboard = (WhiteboardPlayerPlugin)decisionMaker.getPlugin(CommonPluginTerms.PLAYER_WHITEBOARD_PLUGIN, 1.0);
            WBGameMessage foundGame = (WBGameMessage)whiteboard.seekGame(CommonPluginTerms.MEIER_GAME, getVersion());

            if (foundGame == null){
                //decisionMaker benachrichtigen
                decisionMaker.notifyEndOfGame(MeierPlayerDecisionComponent.NO_GAME_FOUND);
                if (textMode) showMessage("No Game found.");
            }else{
                LookupPlayerPlugin lookup = (LookupPlayerPlugin)decisionMaker.getPlugin(CommonPluginTerms.PLAYER_LOOKUP_PLUGIN, 1.0);
                LUGameMessage gameLookupEntry = new LUGameNachricht(foundGame.getGameName(), foundGame.getTuplespace(), null, foundGame.getLeaderID(), getVersion());
                //zum migrieren auffordern
                LUGameMessage foundLookupEntry = lookup.getGame(gameLookupEntry);
                HostLocation gameLocation = foundLookupEntry.getPlace();
                decisionMaker.setMigrationHost(gameLocation);
                leaderID = foundGame.getLeaderID();
                TStoUse = foundGame.getTuplespace();
                decisionMaker.notifyEndOfGame(MeierPlayerDecisionComponent.GAME_DID_NOT_START);
            }
        }else{
            activate(TStoUse);
        }
    }
    
    public void activate(String TSName) {
        if (textMode) showMessage("Activating with TS: "+TSName);
        setCommAgent(CommAgent.load(decisionMaker.getLimeServer()));
        chief.setPluginAndTS(this, TSName);
        playing = true;
        //showMessage("Chief: "+chief+" is actvive "+chief.isActive());
        this.start();
    }
    
    public String getPluginName() {
        return pluginName;
    }
    
    public void run() {
        migratable = false;        
        playing = true;
        while(opQueue == null){
            try{
                sleep(100L);
                if(textMode) showMessage("Warte auf OpQueue");
            }catch (InterruptedException e){
                e.printStackTrace();
            }
        }
        if (opQueue != null){
            if (playing){    
                //Wenn noch nicht eingelogt
                if (textMode) showMessage("Starting to play.");
                if (!loggedOn){
                    ITuple leaderID = terms.generateLeaderIDTuple();
                    ITuple foundID = null;
                    try{
                        if (textMode) showMessage("fetching leader id.");
                        foundID = (ITuple)opQueue.enqueueOp("rd", new Object[]{/*leaderLocation, AgentLocation.UNSPECIFIED,*/ leaderID});
                    }catch (NoSuchMethodException e){
                        showMessage("Exception occured...");
                        e.printStackTrace();
                    }
                    AgentID leader = (AgentID)foundID.get(terms.LEADERID_TUPLE_ID_POSITION).getValue();
                    this.leaderLocation = new AgentLocation(leader);
                    //Einloggen versuchen
                    if (textMode) showMessage("Trying to logon to Game...");
                    //AnmeldungsTuple bauen
                    ITuple anmeldung = terms.generateLogonTuple();
                    IField idField = new Field();
                    //Eigene ID einsetzen
                    idField.setToActual(new Long(decisionMaker.getID()));
                    anmeldung.set(idField, terms.ANMELDUNG_TUPLE_PLAYER_ID_POSITION);
                    //Tuple in TS stellen
                    try{
                        opQueue.enqueueOp("out", new Object[]{leaderLocation, anmeldung});
                    }catch (NoSuchMethodException e){
                        showMessage("Exception occured...");
                        e.printStackTrace();
                    }
                    //Time mit maximaler Wartezeit auf Besttigung starten
                    Timer timer = new Timer(this.timeToWaitForLogon, textMode);
                    timer.start();
                    if (textMode) showMessage("Waiting for acknowledgement.");
                    //Warten auf Besttigung
                    //solange noch nicht eingeloggt und Zeit zum Warten auf Besttigung nicht abgelaufen und logon nicht abgelehnt
                    boolean logonDenied = false;
                    while ( (!loggedOn) && (!timer.TimeElapsed()) && (!logonDenied) ) {
                        ITuple anmeldungsBestaetigung = terms.generateLogonAcknowledgementTuple();
                        anmeldungsBestaetigung.set(idField, terms.ANMELDUNGS_BESTAETIGUNG_PLAYER_ID_POSITION);
                        ITuple bestaetigung = null;
                        //Versuchen die Bestaetigung einzulesen
                        try{
                            bestaetigung = (ITuple)opQueue.enqueueOp("inp", new Object[]{leaderLocation, AgentLocation.UNSPECIFIED, anmeldungsBestaetigung});
                        }catch (NoSuchMethodException e){
                            showMessage("Exception occured...");
                            e.printStackTrace();
                        }
                        //Wenn Bestaetigung da
                        if (bestaetigung != null){
                            //Timer anhalten
                            timer.stopTimer();
                            if (textMode) {
                                showMessage("Tuple found: "+bestaetigung);
                            }
                            boolean logonAccepted = ((Boolean)bestaetigung.get(terms.ANMELDUNGS_BESTAETIGUNG_BESTAETIGUNGS_POSITION).getValue()).booleanValue();
                            if (logonAccepted){
                                //Eingeloggt
                                loggedOn = true;
                                if (textMode) showMessage("logon acknowledged.");
                            }else{
                                playing = false;
                                logonDenied = true;
                                decisionMaker.notifyEndOfGame(MeierPlayerDecisionComponent.GAME_LOGON_DENIED);
                                if (textMode) showMessage("logon denied.");
                            }
                        }
                    }
                    //Wenn Timer abgelaufen,
                    if (timer.TimeElapsed() && !loggedOn){
                        if (textMode) showMessage("Waiting for logon acknowledgement timed out.");
                        //dann "Spielen" beenden
                        decisionMaker.notifyEndOfGame(MeierPlayerDecisionComponent.GAME_LOGON_DENIED);
                        playing = false;
                    }
                }
                //Logon beendet...
                //Reaction zur SpielBeobachtung hinzufuegen
                if (loggedOn){
                    ITuple observeTemplate = terms.generateInformationTuple();
                    //Reaction auf Tuple zur Beendigung eines Spiels hinzufgen
                    ITuple endTemplate = terms.generateEndGameTuple();
                    IField myID = new Field();
                    myID.setToActual(new Long(decisionMaker.getID()));
                    endTemplate.set(myID, terms.END_GAME_PLAYER_ID_POSITION);
                    //Reaction auf Punktevergabe hinzufuegen.
                    ITuple pointTemplate = terms.generatePointAssignmentTuple();
                    pointTemplate.set(myID, terms.POINT_ASSIGNMENT_TUPLE_PLAYER_ID_POSITION);
                    ITuple showGUI = terms.generateShowGUITuple();
                    showGUI.set(myID, terms.SHOW_GUI_TUPLE_PLAYERID_POSITION);
                    Reaction[] react = new Reaction[] { new UbiquitousReaction(observeTemplate, new GameObserverListener(this), Reaction.ONCEPERTUPLE),
                                                        new UbiquitousReaction(pointTemplate, new PointAssignmentListener(this), Reaction.ONCEPERTUPLE),
                                                        new UbiquitousReaction(terms.generatePlayersTuple(), new PlayersListener(this), Reaction.ONCEPERTUPLE),
                                                        new UbiquitousReaction(showGUI, new ShowGUIListener(this), Reaction.ONCEPERTUPLE)
                                                       };
                    try{                                       
                        reactions = (RegisteredReaction[])opQueue.enqueueOp("addWeakReaction", new Object[]{react});
                    }catch (NoSuchMethodException e){
                        showMessage("");
                        e.printStackTrace();
                    }
                    boolean gaming = true;
                    boolean gameHasBegun = false;
                    //Solange gespielt wird
                    while (gaming){
                        try{
                            if (pointAssignmentToFetch != null){
                                try {
                                    ITuple points = (ITuple)opQueue.enqueueOp("inp", new Object[]{leaderLocation, AgentLocation.UNSPECIFIED, pointAssignmentToFetch});
                                }catch (NoSuchMethodException e){
                                    showMessage("");
                                    e.printStackTrace();
                                }
                            }
                            if (textMode) showMessage("Waiting for call to move.");
                            if (gui != null) {
                                gui.setStatus("Warte...");
                                waitOneMoment(slowMoFactor);
                            }
                            ITuple callForMoveTemplate = terms.generateCallForMoveTuple();
                            callForMoveTemplate.set(myID, terms.CALLFORMOVE_TUPLE_PLAYER_ID_POSITION);
                            //Warten auf Aufforderung Zug zu machen
                            ITuple call = (ITuple)opQueue.enqueueOp("in", new Object[]{leaderLocation, AgentLocation.UNSPECIFIED, callForMoveTemplate});
                            if (textMode) showMessage("Call for move received.");
                            if (textMode) showMessage("Received Call-Tuple: "+call);
                            if ( ((String)call.get(terms.CALLFORMOVE_TUPLE_ACTION_POSITION).getValue()).equals(terms.CALLFORMOVE_ACTION_GAMEEND)){
                                gaming = false;
                                if (textMode) showMessage("Game ended...");
                                if (gui != null){
                                    gui.endOfGame();
                                    gui = null;
                                }
                                if (gameHasBegun){
                                    decisionMaker.notifyEndOfGame(MeierPlayerDecisionComponent.GAME_REGULAR_END);
                                }else{
                                    decisionMaker.notifyEndOfGame(MeierPlayerDecisionComponent.GAME_DID_NOT_START);
                                }
                            }
                            //Wenn nicht spiel beendet 
                            if (gaming){
                                gameHasBegun = true;
                                if (gui != null) {
                                    gui.setStatus("Ich bin am Zug.");
                                    waitOneMoment(slowMoFactor);
                                }
                                //und wenn aufforderung Aktion zu whlen
                                if ( ((String)call.get(terms.CALLFORMOVE_TUPLE_ACTION_POSITION).getValue()).equals(terms.CALLFORMOVE_ACTION_INITIAL)){
                                    //ZugEmpfehlung vom StrategyPlugin anfordern
                                    int predecessorPropagation = ((Integer)call.get(terms.CALLFORMOVE_TUPLE_PREDECESSOR_RESULT_POSITION).getValue()).intValue();
                                    String action = strategy.getActingRecommendation(predecessorPropagation);
                                    //Mitteilen, welcher Zug gemacht werden soll
                                    IField myAction = new Field();
                                    myAction.setToActual(action);
                                    ITuple move = terms.generateMoveTuple();
                                    move.set(myID, terms.MOVE_TUPLE_PLAYER_ID_POSITION);
                                    move.set(myAction, terms.MOVE_TUPLE_ACTION_POSITION);
                                    opQueue.enqueueOp("out", new Object[]{this.leaderLocation , move});
                                    //Wenn wuerfeln
                                    if (action.equals(terms.THROWDICE_ACTION)){
                                        if (gui != null) {
                                            gui.showAction("Ich wuerfele.");
                                            waitOneMoment(slowMoFactor);
                                        }
                                       //auf Tuple mit Wuerfel warten
                                        ITuple wuerfelTemplate = terms.generateDiceTuple();
                                        wuerfelTemplate.set(myID, terms.DICE_TUPLE_PLAYER_ID_POSITION);
                                       //wuerfeln
                                        if (textMode) showMessage("Waiting for Dice...");
                                        ITuple wuerfelTuple = (ITuple)opQueue.enqueueOp("in", new Object[]{leaderLocation, AgentLocation.UNSPECIFIED, wuerfelTemplate});
                                        MeierDice dice = (MeierDice)wuerfelTuple.get(terms.DICE_TUPLE_DICE_POSITION).getValue();
                                        if (textMode) showMessage("I 've got the dice.");

                                       //StrategyPlugin fragen, was bekannt gegeben werden soll
                                        int propagation = strategy.getResultToPropagate(predecessorPropagation ,dice.roll());
                                        if (textMode) showMessage("I rolled the dice and i've got: "+terms.diceResultToString(propagation));
                                        if (gui != null) {
                                            gui.showAction("Ich wuerfelte "+terms.diceResultToString(propagation));
                                            waitOneMoment(slowMoFactor);
                                        }
                                        ITuple propagationTuple = terms.generatePropagateResultTuple();
                                        IField gameLeaderID = new Field();
                                        gameLeaderID.setToActual(new Long(leaderID));
                                        IField result = new Field();
                                        result.setToActual(new Integer(propagation));
                                        propagationTuple.set(gameLeaderID, terms.PROPAGATE_RESULT_TUPLE_RECEIVER_ID_POSITION);
                                        propagationTuple.set(result, terms.PROPAGATE_RESULT_TUPLE_RESULT_POSITION);
                                        propagationTuple.set(myID, terms.PROPAGATE_RESULT_TUPLE_SENDER_ID_POSITION);
                                        if (textMode) showMessage("Tuple out: "+propagationTuple);
                                        opQueue.enqueueOp("out", new Object[]{this.leaderLocation, propagationTuple});
                                        if (textMode) showMessage("Result propagated to leader.");

                                       //Wuerfel an GameLeader zurck
                                        wuerfelTuple = terms.generateDiceTuple();
                                        wuerfelTuple.set(gameLeaderID, terms.DICE_TUPLE_PLAYER_ID_POSITION);
                                        IField diceField = new Field();
                                        diceField.setToActual(dice);
                                        wuerfelTuple.set(diceField, terms.DICE_TUPLE_DICE_POSITION);
                                        opQueue.enqueueOp("out", new Object[]{this.leaderLocation, wuerfelTuple});
                                        if (textMode) showMessage("Tuple out: "+wuerfelTuple);
                                        if (textMode) showMessage("Dice passed.");

                                    }else{
                                    //Sonst Aufdecken
                                        if (action.equals(terms.DONT_TRUST_PREDECESSOR)){
                                            if (textMode) showMessage ("I do not trust predecessor.");
                                            if (gui != null) {
                                                gui.showAction("Ich decke die Wuerfel auf.");
                                                waitOneMoment(slowMoFactor);
                                            }
                                           //Passiert nichts Ergebnis kommt ber reactions
                                           //Ergebnis des Zuges entgegennehmen
                                           //Eregbnis dem StrategyPlugin mitteilen
                                        }
                                    }
                                }else{
                                    //sonst wenn Aufforderung Wuerfeln
                                    if ( (((String)call.get(terms.CALLFORMOVE_TUPLE_ACTION_POSITION).getValue()).equals(terms.CALLFORMOVE_ACTION_AFTER_UNCOVER_DICE)) || (((String)call.get(terms.CALLFORMOVE_TUPLE_ACTION_POSITION).getValue()).equals(terms.CALLFORMOVE_ACTION_AFTER_SUCCESSOR_UNCOVERED_DICE)) ){
                                        if (((String)call.get(terms.CALLFORMOVE_TUPLE_ACTION_POSITION).getValue()).equals(terms.CALLFORMOVE_ACTION_AFTER_SUCCESSOR_UNCOVERED_DICE)){ 
                                            strategy.lieWasDetected();
                                            if (gui != null){
                                                gui.setStatus("Ich bin wieder dran.");
                                                gui.showAction("Ich habe gelogen. Wuerfele...");
                                                waitOneMoment(slowMoFactor);
                                            }
                                        }else{
                                            if (gui != null){
                                                gui.setStatus("Ich bin dran.");
                                                gui.showAction("Ich wuerfel.");
                                                waitOneMoment(slowMoFactor);
                                            }
                                        }
                                        //auf Tuple mit Wuerfel warten
                                        ITuple wuerfelTemplate = terms.generateDiceTuple();
                                        wuerfelTemplate.set(myID, terms.DICE_TUPLE_PLAYER_ID_POSITION);
                                       //wuerfeln
                                        if (textMode) showMessage("Waiting for Dice...");
                                        ITuple wuerfelTuple = (ITuple)opQueue.enqueueOp("in", new Object[]{leaderLocation, AgentLocation.UNSPECIFIED, wuerfelTemplate});
                                        MeierDice dice = (MeierDice)wuerfelTuple.get(terms.DICE_TUPLE_DICE_POSITION).getValue();
                                        if (textMode) showMessage("I 've got the dice.");

                                       //StrategyPlugin fragen, was bekannt gegeben werden soll, da erster
                                        int resultOfPredecessor = MeierTerms.PREDECESSOR_RESULT_START;
                                        int propagation = strategy.getResultToPropagate(resultOfPredecessor ,dice.roll());
                                        if (textMode) showMessage("I rolled the dice and i've got: "+terms.diceResultToString(propagation));
                                        ITuple propagationTuple = terms.generatePropagateResultTuple();
                                        IField gameLeaderID = new Field();
                                        gameLeaderID.setToActual(new Long(leaderID));
                                        IField result = new Field();
                                        result.setToActual(new Integer(propagation));
                                        propagationTuple.set(gameLeaderID, terms.PROPAGATE_RESULT_TUPLE_RECEIVER_ID_POSITION);
                                        propagationTuple.set(result, terms.PROPAGATE_RESULT_TUPLE_RESULT_POSITION);
                                        propagationTuple.set(myID, terms.PROPAGATE_RESULT_TUPLE_SENDER_ID_POSITION);
                                        if (textMode) showMessage("Tuple out: "+propagationTuple);
                                        opQueue.enqueueOp("out", new Object[]{this.leaderLocation, propagationTuple});
                                        if (textMode) showMessage("Result propagated to leader.");
                                        if (gui != null){
                                            gui.showAction("Ich habe "+terms.diceResultToString(propagation)+" gewuerfelt.");
                                            waitOneMoment(slowMoFactor);
                                        }

                                       //Wuerfel an GameLeader zurck
                                        wuerfelTuple = terms.generateDiceTuple();
                                        wuerfelTuple.set(gameLeaderID, terms.DICE_TUPLE_PLAYER_ID_POSITION);
                                        IField diceField = new Field();
                                        diceField.setToActual(dice);
                                        wuerfelTuple.set(diceField, terms.DICE_TUPLE_DICE_POSITION);
                                        opQueue.enqueueOp("out", new Object[]{this.leaderLocation, wuerfelTuple});
                                        if (textMode) showMessage("Tuple out: "+wuerfelTuple);
                                        if (textMode) showMessage("Dice passed.");
                                    }
                                }
                            }
                        }catch (NoSuchMethodException e){
                            showMessage("");
                            e.printStackTrace();
                        }
                    }
                    textMode = true;
                    if (textMode) showMessage("I achieved "+this.myPoints+" points.");
                    textMode = false;
                    //deactivate();
                    decisionMaker.notifyEndOfGame(MeierPlayerDecisionComponent.GAME_REGULAR_END);
                }else{
                    if (textMode) showMessage("Logon to Game denied. Giving control back to Deciscionmaker.");
                }
            }else{
                if (observing){
                }
            }
        }
        while (!chief.migrationPossible()){
            waitOneMoment(100);
        }
        chief = null;
        opQueue = null;
        
        
        reactions = null;
    
    
        //MeierTerms terms = null;
        //MeierPlayerStrategyPlugin strategy = null;
        //decisionMaker = null;
        //HostLocation myLocation = null;
        //AgentLocation leaderLocation = null;
        //ITuple pointAssignmentToFetch = null;
        //String TStoUse = null;
        MeierPlayerGUI gui = null;
    
    
        migratable = true;
    }
    
    public GamePlayerStrategyPlugin getStrategy() {
        return strategy;
    }
    
    public void executeQueuedOp() {
        opQueue.executeNextQueuedOp();
    }
    
    public double getVersion() {
        return version;
    }
    
    public void setTupleSpace(lime.LimeTupleSpace TS) {
        showMessage("Create OpQueue");
        this.opQueue = new OpQueue(TS);
        showMessage("OpQueue created");
    }
    
    public void setCommAgent(CommAgent com) {
        this.chief = com;
        myLocation = new HostLocation(decisionMaker.getLimeServer().getServerID());
    }
    
    public boolean isActive(){
        return playing;
    }
    
    public void deactivate(){
        System.out.println("MeierPlayerPlugin: Deactivation-Message received!");
        playing = false;
        observing = false;
        if(opQueue != null){
            try{
                System.out.println("MeierPlayer: removing weakReaction!");
                opQueue.enqueueOp("removeWeakReaction", new Object[]{reactions});
                System.out.println("MeierPlayer: weakReaction removed!");
            }catch(NoSuchMethodException e){
                showMessage("");
                e.printStackTrace();
            }
        }
        /*decisionMaker.getLimeServer().disengage();*/
        /*try{
            sleep(9000L);
        }catch (InterruptedException e){
            showMessage("");
            e.printStackTrace();
        }*/
        if (strategy != null) strategy.deactivate();
        if (chief != null) chief.stopComm();
        if (textMode) showMessage("Chief: "+chief);
    }
    
    public String getTyp(){
        return casino.plugins.CommonPluginTerms.PLAYER_MEIER_PLUGIN;
    }
    
    protected void showMessage(String out) {
        System.out.println("Meier Player "+decisionMaker.getID()+": "+out);
    }
    
    public void setDecisionPlugin(casino.plugins.decision.DecisionPlugIn decision) {
        decisionMaker = decision;
    }
    
    public void play() {
        playing = true;
        observing = false;
    }
    
    public void evaluateGameInformation(ITuple info){
        MeierAction action = new MeierAction();
        action.setPlayerID(((Long)info.get(terms.INFORMATION_TUPLE_ACTING_PLAYER_POSITION).getValue()).longValue());
        action.setAction((String)info.get(terms.INFORMATION_TUPLE_PLAYER_ACTION_POSITION).getValue());
        action.setResultOfAction(((Integer)info.get(terms.INFORMATION_TUPLE_PLAYER_ACTION_RESULT_POSITION).getValue()).intValue());
        if (action.getPlayerID() == this.decisionMaker.getID() ){
            if ( (action.getResultOfAction() == terms.UNCOVERING_DICE_SUCCESFULL ) || (action.getResultOfAction() == terms.UNCOVERING_DICE_UNSUCCESFULL ) ){
                strategy.setResultOfActingRecommendation(action.getResultOfAction());
            }
        }else{
            strategy.setLastObservedAction(action);
        }
    }
    
    public void assignPoints(int points, ITuple pointAssignment){
        myPoints+=points;
        if (gui != null) {
            gui.setPoints(myPoints);
        }
        this.pointAssignmentToFetch = pointAssignment;
        /*try{
            ITuple pointAssign = (ITuple)opQueue.enqueueOp("inp", new Object[]{leaderLocation, AgentLocation.UNSPECIFIED, pointAssignment});
        }catch (NoSuchMethodException e){
            showMessage("Exception occured...");
            e.printStackTrace();
        }*/
    }
    
    public void updatePlayerList(ITuple players){
        /*try{
            ITuple pl = (ITuple)opQueue.enqueueOp("inp", new Object[]{leaderLocation, AgentLocation.UNSPECIFIED, players});
        }catch (NoSuchMethodException e){
            showMessage("Exception occured...");
            e.printStackTrace();
        }*/
    }
    
    public void showGUI(ITuple callToShowGUI){
        /*try{
            ITuple call = (ITuple)opQueue.enqueueOp("inp", new Object[]{leaderLocation, AgentLocation.UNSPECIFIED, callToShowGUI});
        }catch (NoSuchMethodException e){
            showMessage("Exception occured...");
            e.printStackTrace();
        }*/
        if (gui == null){
            gui = new MeierPlayerGUI();
            gui.setObservedPlayer(this);
            gui.setTitle("Meier-Playeragent"+decisionMaker.getID());
            gui.setPoints(this.myPoints);
            gui.show();
        }
    }
    
    public void guiClosed(){
        gui = null;
    }
    
    private void waitOneMoment(long time){
        try{
            sleep(time);
        }catch (InterruptedException e){
            showMessage("");
            e.printStackTrace();
        }
    }
    
    public void setTextMode(boolean textMode) {
        this.textMode = textMode;
    }
    
    public boolean isMigratable(){
        if (textMode) showMessage("Plugin Meier-Player finished: "+migratable);        
        boolean rueckgabe = false;
        if(chief != null){      
            if (textMode) showMessage("Communicator finished: "+chief.migrationPossible());
            rueckgabe = (migratable) && (chief.migrationPossible());
            if(chief.migrationPossible()){
                chief = null;
            }
        }else{
            rueckgabe = migratable;
        }
        return rueckgabe;
    }
    
    public void setSlowMoFactor(long slowMoFactor) {
        this.slowMoFactor = slowMoFactor;
        if (slowMoFactor > 0){
            slowMo = true;
        }else{
            slowMo = false;
        }
    }
    
    public MeierPlayerRulePlugin getNewInstance() {
        MeierPlayer newInstance = new MeierPlayer();
        newInstance.setDecisionPlugin(this.decisionMaker);
        newInstance.setTSName(this.TStoUse);
        newInstance.setLeaderID(this.leaderID);
        newInstance.setStrategy(this.strategy);
        return (MeierPlayerRulePlugin)newInstance;
    }
    
    public void setTSName(java.lang.String name) {
        this.TStoUse = name;
    }
    
    public void setLeaderID(long leaderID) {
        this.leaderID = leaderID;
    }
    
    public void setStrategy(MeierPlayerStrategyPlugin strategy) {
        this.strategy = strategy;
    }
    
    protected class PointAssignmentListener implements ReactionListener{
       
        MeierPlayer player = null;
        
        public PointAssignmentListener(MeierPlayer player){
            this.player = player;
        }
        
        public void reactsTo(lime.ReactionEvent reactionEvent) {
            ITuple pointAssignment = reactionEvent.getEventTuple();
            player.assignPoints(((Integer)pointAssignment.get(terms.POINT_ASSIGNMENT_TUPLE_POINTS_POSITION).getValue()).intValue(), pointAssignment);
        }
        
    }
    
    protected class GameObserverListener implements ReactionListener{
       
        MeierPlayer player = null;
        
        public GameObserverListener(MeierPlayer player){
            this.player = player;
        }
        
        public void reactsTo(lime.ReactionEvent reactionEvent) {
            player.evaluateGameInformation(reactionEvent.getEventTuple());
        }
        
    }
    
    protected class PlayersListener implements ReactionListener {
        
        MeierPlayer player = null;
        
        public PlayersListener(MeierPlayer player){
            this.player = player;
        }
        
        public void reactsTo(lime.ReactionEvent reactionEvent){
            player.updatePlayerList(reactionEvent.getEventTuple());
        }
    }
    
    protected class ShowGUIListener implements ReactionListener {
        
        MeierPlayer player = null;
        
        public ShowGUIListener(MeierPlayer player){
            this.player = player;
        }
        
        public void reactsTo(lime.ReactionEvent reactionEvent) {
            player.showGUI(reactionEvent.getEventTuple());
        }
        
    }
    
}
