/*
 * WhiteboardLeaderComponent.java
 *
 * Created on 23. Juni 2002, 01:31
 */

package casino.plugins.whiteboard;

import lime.*;

import casino.misc.*;
import casino.plugins.*;
import casino.agents.*;
import casino.plugins.whiteboard.Version1_0.*;
import casino.plugins.whiteboard.basic.*;
import casino.plugins.decision.*;
import casino.plugins.lookup.*;

import java.util.Vector;
import java.io.*;

/**
 *
 * @author  Administrator
 * @version 
 */
public class WhiteboardLeaderComponent extends Thread implements WhiteboardLeaderPlugin,Serializable {
    
    protected static final double version = 1.0;
    
    protected LimeTupleSpace lts = null;
    protected String name = null;
    protected OpQueue operationQueue = null;
    protected CommAgent communicator = null;
    protected boolean active = false;
    protected HostLocation host = null;
    protected AgentLocation agentL = null;
    protected DecisionPlugIn leader = null;
    protected Vector openGames = null;
    protected boolean migratable = false;

    /** Creates new WhiteboardLeaderComponent */
    public WhiteboardLeaderComponent(String initialName) {
        name = initialName;
    }
    
    public void run(){
        migratable = false;
        openGames = new Vector();
        if(communicator == null){
            communicator = CommAgent.load(leader.getLimeServer()); 
            communicator.setPluginAndTS(this,CommonPluginTerms.WHITEBOARD_TS_VERSION_1_0);
        }      
        while((operationQueue == null)||(leader == null)){
            try{
                sleep(50L);
            }catch(InterruptedException ie){}
        }        
        LeaderLookupPlugin lookup = (LeaderLookupPlugin)leader.getPlugin(CommonPluginTerms.LEADER_LOOKUP_PLUGIN,1.0);
        casino.plugins.lookup.basic.LUWhiteboardMessage wbm = lookup.getWhiteboard(new casino.plugins.lookup.version1_0.LUWhiteboardNachricht(-42,null,CommonPluginTerms.WHITEBOARD_TS_VERSION_1_0,1.0));
        host = wbm.getPlace();
        casino.plugins.lookup.basic.LUAdministratorMessage am = lookup.getAdmin(new casino.plugins.lookup.version1_0.LUAdministratorNachricht(null,wbm.getAdminID(),null,null));
        agentL = am.getPlace();
        active = true;        
        while(active){
            try{
                sleep(100L);
            }catch(InterruptedException e){}
        }
        for(int i = 0;i < openGames.size();i++){
            try{
                operationQueue.enqueueOp("inp",new Object[]{host,AgentLocation.UNSPECIFIED,((WBGameMessage)openGames.elementAt(i)).getRequestTuple()});         
            }catch(Exception e){e.printStackTrace();}
        }
        migratable = true;
    }

    public double getVersion() {
        return version;
    }
    
    public void deactivate() {
        active = false;
    }
    
    public String getTyp() {
        return CommonPluginTerms.LEADER_WHITEBOARD_PLUGIN;
    }
    
    public void setCommAgent(casino.agents.CommAgent com) {
        communicator = com;
    }
    
    public boolean isActive() {
        return active;
    }
    
    public void activate() {
        this.start();
    }
    
    public void setLeader(DecisionPlugIn newLeader) {
        leader = newLeader;
    }
    
    public void setTupleSpace(LimeTupleSpace TS) {
        lts = TS;
        operationQueue = new OpQueue(lts);
    }
    
    public void executeQueuedOp() {
        operationQueue.executeNextQueuedOp();
    }
    
    public void propagateGame(WBGameMessage game) {      
        WBGameNachricht request = new WBGameNachricht(game.getGameName(), game.getLeaderName(), game.getLeaderID(), game.getFieldID(), game.getVersion(), game.getTuplespace());
        try{
            operationQueue.enqueueOp("out",new Object[]{agentL,request.getResultTuple()});
            openGames.add(game);
        }catch(Exception e){e.printStackTrace();}
    }
    
    public String getPluginName() {
        return name;
    }
    
    public void activate(String TSName){
    }
    
    public boolean unpropagateGame(WBGameMessage game) {
        WBGameNachricht request = new WBGameNachricht(game.getGameName(), game.getLeaderName(), game.getLeaderID(), game.getFieldID(), game.getVersion(), game.getTuplespace());
        try{
            if(operationQueue.enqueueOp("inp",new Object[]{host,AgentLocation.UNSPECIFIED,request.getRequestTuple()}) == null){
                System.out.println("Cant remove game from whiteboard!");
            }
            openGames.remove(request);
        }catch(Exception e){e.printStackTrace();}
        return true;
    }
    
    public boolean isMigratable() {
        return migratable;
    }
    
    protected void showMessage(String out){
        System.out.println("WhiteboardLeaderComponent "+this.leader.getID()+": "+out);
    }
    
    
}
