/*
 * WhiteboardPlayerComponent.java
 *
 * Created on 21. Juni 2002, 22:21
 */

package casino.plugins.whiteboard;

import lime.*;
import lights.interfaces.*;

import casino.misc.*;
import casino.plugins.*;
import casino.agents.*;
import casino.plugins.whiteboard.Version1_0.*;
import casino.plugins.whiteboard.basic.*;
import casino.plugins.decision.*;
import casino.plugins.lookup.*;

import java.io.*;

/**
 *
 * @author  jens
 * @version 
 */
public class WhiteboardPlayerComponent extends Thread implements WhiteboardPlayerPlugin,Serializable {
    
    protected static final double version = 1.0;
    
    protected LimeTupleSpace lts = null;
    protected String name = null;
    protected OpQueue operationQueue = null;
    protected CommAgent communicator = null;
    protected boolean active = false;
    protected HostLocation host = null;
    protected AgentLocation agentL = null;
    protected DecisionPlugIn leader = null;
    protected boolean migratable =false;
    
    /** Creates new WhiteboardPlayerComponent */
    public WhiteboardPlayerComponent(String initialName) {
        name = initialName;
    }
    
    public void run(){
        migratable = false;
        showMessage("Starte PlayerWBPlugin!");
        if(communicator == null){
            communicator = CommAgent.load(leader.getLimeServer()); 
            communicator.setPluginAndTS(this,CommonPluginTerms.WHITEBOARD_TS_VERSION_1_0);
        }       
        while((operationQueue == null)||(leader == null)){
            try{
                showMessage("OpQueue: "+operationQueue+" & DecisionComponent: "+leader);
                sleep(50L);
            }catch(InterruptedException ie){}
        }
        showMessage("PlayerWBPlugin: Erfrage Lookup-Plugin!");
        PlayerLookupPlugin lookup = (PlayerLookupPlugin)leader.getPlugin(CommonPluginTerms.PLAYER_LOOKUP_PLUGIN,1.0);
        casino.plugins.lookup.basic.LUWhiteboardMessage wbm = lookup.getWhiteboard(new casino.plugins.lookup.version1_0.LUWhiteboardNachricht(-42,null,CommonPluginTerms.WHITEBOARD_TS_VERSION_1_0,1.0));
        showMessage("PlayerWBPlugin: Setze Host!");
        host = wbm.getPlace();
        casino.plugins.lookup.basic.LUAdministratorMessage am = lookup.getAdmin(new casino.plugins.lookup.version1_0.LUAdministratorNachricht(null,wbm.getAdminID(),null,null));
        showMessage("PlayerWBPlugin: Setze AgentLocation!");
        agentL = am.getPlace();
        
        active = true;
        showMessage("PlayerWBPlugin: Aktiviert!");
        while(active){
            try{
                sleep(100L);
            }catch(InterruptedException e){}
        }
        communicator.stopComm();
        migratable = true;
        active = false;
    }

    public WBGameMessage seekGame(String gameName, double gameVersion) {
        WBGameNachricht message = new WBGameNachricht(gameName,null,-42,-42,gameVersion,null);
        WBGameMessage answer = null;
        try{
            ITuple test = (ITuple)operationQueue.enqueueOp("rdp",new Object[]{host,AgentLocation.UNSPECIFIED,message.getRequestTuple()});
            if(test != null){
                answer = new WBGameNachricht(test);
            }
        }catch(Exception e){e.printStackTrace();showMessage("LimeFehler!");}
        if(answer == null){
            return null;
        }
        return answer;
    }
    
    public String getTyp() {
        return CommonPluginTerms.PLAYER_WHITEBOARD_PLUGIN;
    }
    
    public String getPluginName() {
        return name;
    }
    
    public void activate() {        
        this.start();
    }
    
    public void activate(String TSName){
        
    }
    
    public void setTupleSpace(LimeTupleSpace TS) {
        lts = TS;
        operationQueue = new OpQueue(lts);
    }
    
    public void executeQueuedOp() {
        operationQueue.executeNextQueuedOp();
    }
    
    public void setCommAgent(casino.agents.CommAgent com) {
        communicator = com;
    }
    
    public double getVersion() {
        return version;
    }
    
    public void deactivate() {
        System.out.println("WBPlayerPlugin: Deactivation-Message received!");
        active = false;
    }
    
    public boolean isActive() {
        return active;
    }
    
    public void setLeader(DecisionPlugIn newLeader) {
        leader = newLeader;
    }
    
    public WBGameMessage[] getAllGames() {
        WBGameNachricht message = new WBGameNachricht(null,null,-42,-42,-42,null);
        WBGameMessage[] answers = null;
        try{
            ITuple[] tuples = (ITuple[])operationQueue.enqueueOp("rdg",new Object[]{host,null,message.getRequestTuple()});
            answers = new WBGameMessage[tuples.length];
            for(int i = 0;i < tuples.length;i++){
                answers[i] = new WBGameNachricht(tuples[i]);
            }
        }catch(Exception e){e.printStackTrace();showMessage("LimeFehler!");}
        return answers;
    }
    
    public boolean isMigratable() {
        //showMessage("Plugin finished: "+migratable);        
        boolean rueckgabe = false;
        if(communicator != null){      
            showMessage("Communicator finished: "+communicator.migrationPossible());
            rueckgabe = (migratable) && (communicator.migrationPossible());
            if(communicator.migrationPossible()){
                communicator = null;
            }
        }else{
            rueckgabe = migratable;
        }
        return rueckgabe;
    }
    
    protected void showMessage(String out){
        System.out.println("WhiteboardPlayerComponent "+this.leader.getID()+": "+out);
    }
    
}
